# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from enum import IntEnum

from .LibraryUtility import AppStoreServerLibraryEnumMeta

class ExtendReasonCode(IntEnum, metaclass=AppStoreServerLibraryEnumMeta):
    """
    The code that represents the reason for the subscription-renewal-date extension.
    
    https://developer.apple.com/documentation/appstoreserverapi/extendreasoncode
    """
    UNDECLARED = 0
    CUSTOMER_SATISFACTION = 1
    OTHER = 2
    SERVICE_ISSUE_OR_OUTAGE = 3



# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

import pathlib

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="app-store-server-library",
    version="1.0.0",
    description="The App Store Server Library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests"]),
    python_requires=">=3.7, <4",
    install_requires=["attrs >= 21.3.0", 'PyJWT >= 2.6.0, < 3', 'requests >= 2.28.0, < 3', 'cryptography >= 40.0.0, < 42', 'pyOpenSSL >= 23.1.1, < 24', 'asn1==2.7.0', 'cattrs==23.1.2'],
    package_data={"appstoreserverlibrary": ["py.typed"]},
)



# Copyright (c) 2023 Apple Inc. Licensed under MIT License.
from typing import Optional

from attr import define
import attr

@define
class ExtendRenewalDateResponse: 
    """
    A response that indicates whether an individual renewal-date extension succeeded, and related details.
    
    https://developer.apple.com/documentation/appstoreserverapi/extendrenewaldateresponse
    """

    originalTransactionId: Optional[str] = attr.ib(default=None)
    """
    The original transaction identifier of a purchase.
    
    https://developer.apple.com/documentation/appstoreserverapi/originaltransactionid
    """

    webOrderLineItemId: Optional[str] = attr.ib(default=None)
    """
    The unique identifier of subscription-purchase events across devices, including renewals.
    
    https://developer.apple.com/documentation/appstoreserverapi/weborderlineitemid
    """

    success: Optional[bool] = attr.ib(default=None)
    """
    A Boolean value that indicates whether the subscription-renewal-date extension succeeded.
    
    https://developer.apple.com/documentation/appstoreserverapi/success
    """

    effectiveDate: Optional[int] = attr.ib(default=None)
    """
    The new subscription expiration date for a subscription-renewal extension.
    
    https://developer.apple.com/documentation/appstoreserverapi/effectivedate
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.hashes import SHA256
from cryptography.hazmat.primitives.asymmetric.ec import ECDSA, EllipticCurvePrivateKey

import uuid
import base64

class PromotionalOfferSignatureCreator:
    _signing_key: EllipticCurvePrivateKey
    _key_id: str
    _bundle_id: str
    def __init__(self, signing_key: bytes, key_id: str, bundle_id: str):
       self._signing_key = serialization.load_pem_private_key(signing_key, password=None, backend=default_backend())
       self._key_id = key_id
       self._bundle_id = bundle_id
    def create_signature(self, product_identifier: str, subscription_offer_id: str, application_username: str, nonce: uuid.UUID, timestamp: int):
        """
        Return the Base64 encoded signature

        https://developer.apple.com/documentation/storekit/in-app_purchase/original_api_for_in-app_purchase/subscriptions_and_offers/generating_a_signature_for_promotional_offers

        :param product_identifier: The subscription product identifier
        :param subscription_offer_id: The subscription discount identifier
        :param application_username: An optional string value that you define; may be an empty string
        :param nonce: A one-time UUID value that your server generates. Generate a new nonce for every signature.
        :param timestamp: A timestamp your server generates in UNIX time format, in milliseconds. The timestamp keeps the offer active for 24 hours.
        :return: The Base64 encoded signature
        """
        payload = self._bundle_id + '\u2063' + \
            self._key_id + '\u2063' + \
            product_identifier + '\u2063' + \
            subscription_offer_id + '\u2063' + \
            application_username.lower()  + '\u2063'+ \
            str(nonce).lower() + '\u2063' + \
            str(timestamp)

        return base64.b64encode(self._signing_key.sign(
            payload.encode('utf-8'), ECDSA(SHA256())
        )).decode('utf-8')



# Copyright (c) 2023 Apple Inc. Licensed under MIT License.
from typing import Optional

from attr import define
import attr

@define
class MassExtendRenewalDateResponse: 
    """
    A response that indicates the server successfully received the subscription-renewal-date extension request.
    
    https://developer.apple.com/documentation/appstoreserverapi/massextendrenewaldateresponse
    """

    requestIdentifier: Optional[str] = attr.ib(default=None)
    """
    A string that contains a unique identifier you provide to track each subscription-renewal-date extension request.
    
    https://developer.apple.com/documentation/appstoreserverapi/requestidentifier
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.
from typing import List, Optional

from attr import define
import attr

from .SendAttemptItem import SendAttemptItem

@define
class CheckTestNotificationResponse: 
    """
    A response that contains the contents of the test notification sent by the App Store server and the result from your server.
    
    https://developer.apple.com/documentation/appstoreserverapi/checktestnotificationresponse
    """

    signedPayload: Optional[str] = attr.ib(default=None)
    """
    A cryptographically signed payload, in JSON Web Signature (JWS) format, containing the response body for a version 2 notification.
    
    https://developer.apple.com/documentation/appstoreservernotifications/signedpayload
    """

    sendAttempts: Optional[List[SendAttemptItem]] = attr.ib(default=None)
    """
    An array of information the App Store server records for its attempts to send the TEST notification to your server. The array may contain a maximum of six sendAttemptItem objects.
    
    https://developer.apple.com/documentation/appstoreserverapi/sendattemptitem
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from attr import define
from typing import List, Optional
import attr

from .Environment import Environment
from .LibraryUtility import AttrsRawValueAware

@define
class HistoryResponse(AttrsRawValueAware):
    """
    A response that contains the customer's transaction history for an app.
    
    https://developer.apple.com/documentation/appstoreserverapi/historyresponse
    """

    revision: Optional[str] = attr.ib(default=None)
    """
    A token you use in a query to request the next set of transactions for the customer.
    
    https://developer.apple.com/documentation/appstoreserverapi/revision
    """

    hasMore: Optional[bool] = attr.ib(default=None)
    """
    A Boolean value indicating whether the App Store has more transaction data.
    
    https://developer.apple.com/documentation/appstoreserverapi/hasmore
    """

    bundleId: Optional[str] = attr.ib(default=None)
    """
    The bundle identifier of an app.
    
    https://developer.apple.com/documentation/appstoreserverapi/bundleid
    """

    appAppleId: Optional[int] = attr.ib(default=None)
    """
    The unique identifier of an app in the App Store.
    
    https://developer.apple.com/documentation/appstoreservernotifications/appappleid
    """

    environment: Optional[Environment] = Environment.create_main_attr('rawEnvironment')
    """
    The server environment in which you're making the request, whether sandbox or production.
    
    https://developer.apple.com/documentation/appstoreserverapi/environment
    """

    rawEnvironment: Optional[str] = Environment.create_raw_attr('environment')
    """
    See environment
    """

    signedTransactions: Optional[List[str]] = attr.ib(default=None)
    """
    An array of in-app purchase transactions for the customer, signed by Apple, in JSON Web Signature format.
    
    https://developer.apple.com/documentation/appstoreserverapi/jwstransaction
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from attr import define
from typing import List, Optional
import attr

@define
class RefundHistoryResponse: 
    """
    A response that contains an array of signed JSON Web Signature (JWS) refunded transactions, and paging information.
    
    https://developer.apple.com/documentation/appstoreserverapi/refundhistoryresponse
    """

    signedTransactions: Optional[List[str]] = attr.ib(default=None)
    """
    A list of up to 20 JWS transactions, or an empty array if the customer hasn't received any refunds in your app. The transactions are sorted in ascending order by revocationDate.
    """

    revision: Optional[str] = attr.ib(default=None)
    """
    A token you use in a query to request the next set of transactions for the customer.
    
    https://developer.apple.com/documentation/appstoreserverapi/revision
    """

    hasMore: Optional[bool] = attr.ib(default=None)
    """
    A Boolean value indicating whether the App Store has more transaction data.
    
    https://developer.apple.com/documentation/appstoreserverapi/hasmore
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.
from typing import Optional

from attr import define
import attr

from .OfferDiscountType import OfferDiscountType
from .Environment import Environment
from .InAppOwnershipType import InAppOwnershipType
from .LibraryUtility import AttrsRawValueAware
from .OfferType import OfferType
from .RevocationReason import RevocationReason
from .TransactionReason import TransactionReason

from .Type import Type

@define
class JWSTransactionDecodedPayload(AttrsRawValueAware):
    """
    A decoded payload containing transaction information.
    
    https://developer.apple.com/documentation/appstoreserverapi/jwstransactiondecodedpayload
    """

    originalTransactionId: Optional[str] = attr.ib(default=None)
    """
    The original transaction identifier of a purchase.
    
    https://developer.apple.com/documentation/appstoreserverapi/originaltransactionid
    """

    transactionId: Optional[str] = attr.ib(default=None)
    """
    The unique identifier for a transaction such as an in-app purchase, restored in-app purchase, or subscription renewal.
    
    https://developer.apple.com/documentation/appstoreserverapi/transactionid
    """

    webOrderLineItemId: Optional[str] = attr.ib(default=None)
    """
    The unique identifier of subscription-purchase events across devices, including renewals.
    
    https://developer.apple.com/documentation/appstoreserverapi/weborderlineitemid
    """

    bundleId: Optional[str] = attr.ib(default=None)
    """
    The bundle identifier of an app.
    
    https://developer.apple.com/documentation/appstoreserverapi/bundleid
    """

    productId: Optional[str] = attr.ib(default=None)
    """
    The unique identifier for the product, that you create in App Store Connect.
    
    https://developer.apple.com/documentation/appstoreserverapi/productid
    """    

    subscriptionGroupIdentifier: Optional[str] = attr.ib(default=None)
    """
    The identifier of the subscription group that the subscription belongs to.
    
    https://developer.apple.com/documentation/appstoreserverapi/subscriptiongroupidentifier
    """

    purchaseDate: Optional[int] = attr.ib(default=None)
    """
    The time that the App Store charged the user's account for an in-app purchase, a restored in-app purchase, a subscription, or a subscription renewal after a lapse.
    
    https://developer.apple.com/documentation/appstoreserverapi/purchasedate
    """

    originalPurchaseDate: Optional[int] = attr.ib(default=None)
    """
    The purchase date of the transaction associated with the original transaction identifier.
    
    https://developer.apple.com/documentation/appstoreserverapi/originalpurchasedate
    """

    expiresDate: Optional[int] = attr.ib(default=None)
    """
    The UNIX time, in milliseconds, an auto-renewable subscription expires or renews.
    
    https://developer.apple.com/documentation/appstoreserverapi/expiresdate
    """

    quantity: Optional[int] = attr.ib(default=None)
    """
    The number of consumable products purchased.
    
    https://developer.apple.com/documentation/appstoreserverapi/quantity
    """

    type: Optional[Type] = Type.create_main_attr('rawType')
    """
    The type of the in-app purchase.
    
    https://developer.apple.com/documentation/appstoreserverapi/type
    """

    rawType: Optional[str] = Type.create_raw_attr('type')
    """
    See type
    """

    appAccountToken: Optional[str] = attr.ib(default=None)
    """
    The UUID that an app optionally generates to map a customer's in-app purchase with its resulting App Store transaction.
    
    https://developer.apple.com/documentation/appstoreserverapi/appaccounttoken
    """

    inAppOwnershipType: Optional[InAppOwnershipType] = InAppOwnershipType.create_main_attr('rawInAppOwnershipType')
    """
    A string that describes whether the transaction was purchased by the user, or is available to them through Family Sharing.
    
    https://developer.apple.com/documentation/appstoreserverapi/inappownershiptype
    """

    rawInAppOwnershipType: Optional[str] = InAppOwnershipType.create_raw_attr('inAppOwnershipType')
    """
    See inAppOwnershipType
    """

    signedDate: Optional[int] = attr.ib(default=None)
    """
    The UNIX time, in milliseconds, that the App Store signed the JSON Web Signature data.
    
    https://developer.apple.com/documentation/appstoreserverapi/signeddate
    """

    revocationReason: Optional[RevocationReason] = RevocationReason.create_main_attr('rawRevocationReason')
    """
    The reason that the App Store refunded the transaction or revoked it from family sharing.
    
    https://developer.apple.com/documentation/appstoreserverapi/revocationreason
    """

    rawRevocationReason: Optional[int] = RevocationReason.create_raw_attr('revocationReason')
    """
    See revocationReason
    """

    revocationDate: Optional[int] = attr.ib(default=None)
    """
    The UNIX time, in milliseconds, that Apple Support refunded a transaction.
    
    https://developer.apple.com/documentation/appstoreserverapi/revocationdate
    """    

    isUpgraded: Optional[bool] = attr.ib(default=None)
    """
    The Boolean value that indicates whether the user upgraded to another subscription.
    
    https://developer.apple.com/documentation/appstoreserverapi/isupgraded
    """

    offerType: Optional[OfferType] = RevocationReason.create_main_attr('rawOfferType')
    """
    A value that represents the promotional offer type.
    
    https://developer.apple.com/documentation/appstoreserverapi/offertype
    """

    rawOfferType: Optional[int] = OfferType.create_raw_attr('offerType')
    """
    See offerType
    """

    offerIdentifier: Optional[str] = attr.ib(default=None)
    """
    The identifier that contains the promo code or the promotional offer identifier.
    
    https://developer.apple.com/documentation/appstoreserverapi/offeridentifier
    """

    environment: Optional[Environment] = Environment.create_main_attr('rawEnvironment')
    """
    The server environment, either sandbox or production.
    
    https://developer.apple.com/documentation/appstoreserverapi/environment
    """

    rawEnvironment: Optional[str] = Environment.create_raw_attr('environment')
    """
    See environment
    """

    storefront: Optional[str] = attr.ib(default=None)
    """
    The three-letter code that represents the country or region associated with the App Store storefront for the purchase.
    
    https://developer.apple.com/documentation/appstoreserverapi/storefront
    """

    storefrontId: Optional[str] = attr.ib(default=None)
    """
    An Apple-defined value that uniquely identifies the App Store storefront associated with the purchase.
    
    https://developer.apple.com/documentation/appstoreserverapi/storefrontid
    """

    transactionReason: Optional[TransactionReason] = TransactionReason.create_main_attr('rawTransactionReason')
    """
    The reason for the purchase transaction, which indicates whether it's a customer's purchase or a renewal for an auto-renewable subscription that the system initates.
    
    https://developer.apple.com/documentation/appstoreserverapi/transactionreason
    """

    rawTransactionReason: Optional[str] = TransactionReason.create_raw_attr('transactionReason')
    """
    See transactionReason
    """

    currency: Optional[str] = attr.ib(default=None)
    """
    The three-letter ISO 4217 currency code for the price of the product.
    
    https://developer.apple.com/documentation/appstoreserverapi/currency
    """

    price: Optional[int] = attr.ib(default=None)
    """
    The price of the in-app purchase or subscription offer that you configured in App Store Connect, as an integer.
    
    https://developer.apple.com/documentation/appstoreserverapi/price
    """

    offerDiscountType: Optional[OfferDiscountType] = OfferDiscountType.create_main_attr('rawOfferDiscountType')
    """
    The payment mode you configure for an introductory offer, promotional offer, or offer code on an auto-renewable subscription.
    
    https://developer.apple.com/documentation/appstoreserverapi/offerdiscounttype
    """

    rawOfferDiscountType: Optional[str] = OfferDiscountType.create_raw_attr('offerDiscountType')
    """
    See offerDiscountType
    """


# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from enum import IntEnum

from .LibraryUtility import AppStoreServerLibraryEnumMeta

class PlayTime(IntEnum, metaclass=AppStoreServerLibraryEnumMeta):
    """
    A value that indicates the amount of time that the customer used the app.
    
    https://developer.apple.com/documentation/appstoreserverapi/playtime
    """
    UNDECLARED = 0
    ZERO_TO_FIVE_MINUTES = 1
    FIVE_TO_SIXTY_MINUTES = 2
    ONE_TO_SIX_HOURS = 3
    SIX_HOURS_TO_TWENTY_FOUR_HOURS = 4
    ONE_DAY_TO_FOUR_DAYS = 5
    FOUR_DAYS_TO_SIXTEEN_DAYS = 6
    OVER_SIXTEEN_DAYS = 7



# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from enum import Enum
from .LibraryUtility import AppStoreServerLibraryEnumMeta

class Type(str, Enum, metaclass=AppStoreServerLibraryEnumMeta):
    """
    The type of in-app purchase products you can offer in your app.
    
    https://developer.apple.com/documentation/appstoreserverapi/type
    """
    AUTO_RENEWABLE_SUBSCRIPTION = "Auto-Renewable Subscription"
    NON_CONSUMABLE = "Non-Consumable"
    CONSUMABLE = "Consumable"
    NON_RENEWING_SUBSCRIPTION ="Non-Renewing Subscription"



# Copyright (c) 2023 Apple Inc. Licensed under MIT License.

from attr import define
from typing import List, Optional
import attr

from .ExtendReasonCode import ExtendReasonCode

@define
class MassExtendRenewalDateRequest: 
    """
    The request body that contains subscription-renewal-extension data to apply for all eligible active subscribers.
    
    https://developer.apple.com/documentation/appstoreserverapi/massextendrenewaldaterequest
    """

    extendByDays: Optional[int] = attr.ib(default=None)
    """
    The number of days to extend the subscription renewal date.
    
    https://developer.apple.com/documentation/appstoreserverapi/extendbydays
    maximum: 90
    """

    extendReasonCode: Optional[ExtendReasonCode] = attr.ib(default=None)
    """
    The reason code for the subscription-renewal-date extension.
    
    https://developer.apple.com/documentation/appstoreserverapi/extendreasoncode
    """    

    requestIdentifier: Optional[str] = attr.ib(default=None)
    """
    A string that contains a unique identifier you provide to track each subscription-renewal-date extension request.
    
    https://developer.apple.com/documentation/appstoreserverapi/requestidentifier
    """

    storefrontCountryCodes: Optional[List[str]] = attr.ib(default=None)
    """
    A list of storefront country codes you provide to limit the storefronts for a subscription-renewal-date extension.
    
    https://developer.apple.com/documentation/appstoreserverapi/storefrontcountrycodes
    """

    productId: Optional[str] = attr.ib(default=None)
    """
    The unique identifier for the product, that you create in App Store Connect.
    
    https://developer.apple.com/documentation/appstoreserverapi/productid
    """